# -*- coding: utf-8 -*-
"""
Convert mesh tables in case of shema changes

USAGE
    python -m thyrsis.database.convert_mesh_tables [-h, -u, -o, --reproj, --out_srid]

OPTIONS
    -h, --help
        print this help

    -u, --update
        update mesh tables with lastest shema

    -r, --reproj [database path]

    -o, --out_srid [EPSG code], default : 4326

    copy mesh.sqlite in a new database named mesh_saved.sqlite.
    if mesh_saved.sqlite already exists, it will be replaced.
    Then modify mesh.sqlite.
"""

import os
import shutil
from ..database import sqlite
from ..database import create_mesh_tables
from ..settings import Settings
from ..log import logger

def update_mesh_tables():
    """ Update old mesh tables to new shema

    :return: None
    """

    print("adding table openfoam_hynverse_options")

    settings = Settings()

    mesh_dir = settings.value("General", "defaultMeshDir")
    meshdb_list = os.listdir(mesh_dir)

    for meshdb in meshdb_list:

        if meshdb[-12:] != '.mesh.sqlite' or 'saved' in meshdb:
            continue

        print("updating", os.path.join(mesh_dir, meshdb))
        dst = sqlite.connect(os.path.join(mesh_dir, meshdb))
        cur = dst.cursor()

        cur.execute("SELECT name FROM sqlite_master WHERE type='table'")

        if "openfoam_hynverse_options" not in [ x[0] for x in cur.fetchall()]:
            cur.execute("""
                CREATE TABLE openfoam_hynverse_options(
                    id INTEGER PRIMARY KEY,
                    fluid_density REAL DEFAULT 1.e3,
                    dynamic_viscosity REAL DEFAULT 1.e-3,
                    potential_tolerance REAL DEFAULT 1.e-8,
                    potential_tolerance_relative REAL DEFAULT 0.1,
                    potential_residualControl REAL DEFAULT 1.e-6,
                    potential_relaxationFactor REAL DEFAULT 0.5,
                    hwatermin REAL DEFAULT 0.1)""")
            cur.execute("""
                INSERT INTO openfoam_hynverse_options DEFAULT VALUES""")

            dst.commit()

def add_vge_parameters():
    """ Update old mesh tables to new shema
    
    :return: None
    """

    # fix_print_with_import
    print("adding VGE in tables parametres_site")

    settings = Settings()

    meshdb_list = os.listdir(settings.value("General", "defaultMeshDir"))
    
    for meshdb in meshdb_list:

        if meshdb[-7:] != '.sqlite' or 'saved' in meshdb:
            continue

        meshdb_saved = meshdb[:-7]+"_saved.sqlite"
        # fix_print_with_import
        print("copying spatialite database ", meshdb, " into ", meshdb_saved)
        shutil.copyfile(meshdb, meshdb_saved)

        dst = sqlite.connect(meshdb)
        cur = dst.cursor()
        cur.execute("DROP table parametres_site")

        cur.execute("""
            CREATE TABLE parametres_site (
                id INTEGER PRIMARY KEY,
                WC real not null,
                VGA real not null,
                VGN real not null,
                VGR real not null,
                VGS real not null,
                VGE real not null,
                WT real not null,
                VM real not null,
                DLZNS real not null,
                DLZS real not null,
                WM real not null)""")
        cur.execute("""
            INSERT INTO parametres_site(WC, VGA, VGN, VGR, VGS, VGE, WT, VM, DLZNS, DLZS, WM)
                VALUES (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)""")

        dst.commit()

def reproj_mesh_db(meshdb, out_srid):
    """Transform projection of the input mesh database

    :param meshdb: input db path
    :type meshdb: string
    :param out_srid: epsg code
    :type our_srid: integer
    """

    meshdb_saved = meshdb[:-12]+"_saved.mesh.sqlite"

    # fix_print_with_import
    print("copying spatialite database ", meshdb, " into ", meshdb_saved)
    shutil.copyfile(meshdb, meshdb_saved)

    with sqlite.connect(meshdb, "") as src:
        src_cur = src.cursor()

        tmp_db = meshdb[:-12]+"_tmp.mesh.sqlite"
        if os.path.exists(tmp_db):
            os.remove(tmp_db)
        with sqlite.connect(tmp_db, "") as dst:
            dst_cur = dst.cursor()
            dst_cur.execute("SELECT InitSpatialMetaData(1)")
            create_mesh_tables(dst_cur, project_SRID=out_srid)
            dst.commit()

            tables=['hynverse_erreurs','mailles_second_milieu','noeuds_contour', 'noeuds_second_milieu','potentiel_impose', 'zones', 'metis_hynverse_options', 'parametres', 'hynverse_parametres']
            update_tables = ['metis_hynverse_options', 'parametres', 'hynverse_parametres']
            for t in tables:
                if t in update_tables :
                    dst_cur.execute("DELETE FROM {}".format(t))
                    dst.commit()
                rows = src_cur.execute("SELECT * FROM {}".format(t)).fetchall()
                fields = [i[0] for i in src_cur.execute("""SELECT name FROM pragma_table_info('{}')""".format(t)).fetchall()]
                fields_str = ','.join(fields)
                dst_cur.executemany("INSERT INTO {}({}) VALUES ({})".format(t,fields_str,','.join(['?' for i in fields])), rows)
            dst.commit()
            gtables=['bords', 'contours', 'domaines', 'extremites', 'mailles', 'noeuds', 'points_fixes', 'points_pilote']
            for g in gtables:
                gfields = [i[0].upper() for i in src_cur.execute("SELECT f_geometry_column FROM geometry_columns WHERE f_table_name LIKE '{}'".format(g)).fetchall()]
                fields = [i[0] for i in src_cur.execute("""SELECT name FROM pragma_table_info('{}')""".format(g)).fetchall() if i[0] not in gfields]
                fields_str = ','.join(fields)
                gfields_str = ','.join(gfields)
                trans_gfields_str = ','.join(['ST_TRANSFORM({},{}) as {}'.format(i,str(out_srid),i) for i in gfields])
                rows = src_cur.execute("SELECT {},{} FROM {}".format(fields_str,trans_gfields_str,g)).fetchall()
                dst_cur.executemany("INSERT INTO {}({},{}) VALUES ({})".format(g,fields_str,gfields_str,','.join(['?' for i in fields+gfields])), rows)

            dst.commit()

    os.remove(meshdb)
    os.rename(tmp_db,meshdb)


# run as script if invoqued as such
if __name__ == "__main__":
    import sys
    import getopt
    import sip
    assert(sip.getapi('QVariant') == 2)

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                "r:huo:",
                ["reproj=", "out_srid=", "update", "help"])
    except Exception as e:
        sys.stderr.write(str(e)+"\n")
        exit(1)
    optlist = dict(optlist)

    if "-h" in optlist or "--help" in optlist:
        help(sys.modules[__name__])
        exit(0)

    if "-u" in optlist or "--update" in optlist:
        update_mesh_tables()

    if "-r" in optlist or "--reproj" in optlist :
        if "-o" in optlist :
            out_srid=int(optlist["-o"])
        elif "--out_srid" in optlist :
            out_srid=int(optlist["--out_srid"])
        else :
            out_srid = 4326
        db_src = optlist["-r"] if "-r" in optlist else optlist["--reproj"]
        reproj_mesh_db(db_src, out_srid)

    if len(sys.argv) not in [1]:
        logger.error("wrong number of arguments (try compute.py --help)")
        exit(1)
